# Loading module:
# 1. archive.yml
# 2. recipe.yml
# 3. Load_decadal.R (V)

library(yaml)
library(startR)

  dict <- read_yaml("/esarchive/scratch/aho/git/auto-s2s/conf/archive_decadal.yml")
  recipe <- read_yaml("/esarchive/scratch/aho/git/auto-s2s/modules/data_load/recipe_decadal.yml")

  # Read from recipe:
  model <- recipe$Analysis$Datasets$System$name #'HadGEM3'
  member <- strsplit(recipe$Analysis$Datasets$System$member, ',')[[1]] #c("r1i1p1f2", "r2i1p1f2")
  var <- recipe$Analysis$Variables$name #'tas'
  freq <- recipe$Analysis$Variables$freq #monthly_mean
  lat_min <- recipe$Analysis$Region$latmin #0
  lat_max <- recipe$Analysis$Region$latmax #10
  lon_min <- recipe$Analysis$Region$lonmin #0
  lon_max <- recipe$Analysis$Region$lonmax #10
  sdates <- as.numeric(recipe$Analysis$Time$sdate$start):as.numeric(recipe$Analysis$Time$sdate$end) #1960:2015
#  fyear <- as.numeric(recipe$Analysis$Time$fyear$start):as.numeric(recipe$Analysis$Time$fyear$end) #1:1
  time <- as.numeric(recipe$Analysis$Time$leadtimemin):as.numeric(recipe$Analysis$Time$leadtimemax)
  # only used for daily data
  season <- recipe$Analysis$Time$season

  # Read from archive:
  table <- dict$archive$Decadal[[model]][[freq]]$table[[var]]  #'Amon' # depend on var
  grid <- dict$archive$Decadal[[model]][[freq]]$grid[[var]]
  version <- dict$archive$Decadal[[model]][[freq]]$version[[var]]
  if (member == 'all') {
    member <- strsplit(dict$archive$Decadal[[model]]$member, ',')[[1]]
  }

  # derived from above:
#  circularsort <- ifelse(lon_min >= 0, CircularSort(0, 360), CircularSort(-180, 180))
  if (lon_min >= 0) {
    circularsort <- CircularSort(0, 360)
  } else {
    circularsort <- CircularSort(-180, 180)
  }
  
#-------------------------------------------
# Step 1: Load the exp
#-------------------------------------------
#monthly and daily
  path <- file.path(dict$archive$src, dict$archive$Decadal[[model]]$src, 
                    '$member$', table, '$var$', grid, version)
  files <- paste0('$var$_', table, '_*_*_s$sdate$-$member$_', grid, '_$chunk$.nc')

#monthly
#  fyears <- 1
#  fmonths <- (1 + 12 * (fyears[1] - 1)) : (2 + 12 * fyears[length(fyears)])
#daily
#  fdays <- get_days_dcpp(fyears = fyears, season = season, init = 'Nov', calendar = '360-days')

#monthly
  exp <- Start(dat = file.path(path, files),
               var = var,
               member = member,
               sdate = paste0(sdates),
               chunk = 'all',
               chunk_depends = 'sdate',
               fmonth = indices(time),
               fmonth_across = 'chunk',
               merge_across_dims = TRUE,
               lat = values(list(lat_min, lat_max)),
               lat_reorder = Sort(),
               lon = values(list(lon_min, lon_max)),
               lon_reorder = circularsort,
               transform = CDORemapper,
               transform_extra_cells = 2,
               transform_params = list(grid = 'r360x181',  #need to change
                                       method = recipe$Analysis$Regrid$method),
               transform_vars = c('lat', 'lon'),
               synonims = list(fmonth = c('fmonth', 'time'),
                               lon = c('lon', 'longitude'),
                               lat = c('lat', 'latitude')),
               return_vars = list(lat = NULL, lon = NULL, 
                                  fmonth = c('sdate', 'chunk')),
               retrieve = F)



#daily (not done yet)
  data <- Start(dat = path,
                var = variable,
                sdate = paste0(sdates),
                aux = 'all',
                aux_depends = 'sdate',
                lat = values(list(lat_min, lat_max)),
                lat_reorder = Sort(),
                lon = values(list(lon_min, lon_max)),
                lon_reorder = CircularSort(0, 360),
                fday = indices(fdays),
                member = members,
                synonims = list(fday = c('fday', 'time'),
                               lon = c('lon', 'longitude'),
                               lat = c('lat', 'latitude')),
                return_vars = list(lat = NULL, lon = NULL,
                                   fday = c('sdate', 'aux')),
                retrieve = F)



#-------------------------------------------
# Step 2. Load the reference
#-------------------------------------------
path <- '/esarchive/obs/noaa/ghcn_v2/monthly_mean/$var$/$var$_$year$$month$.nc'

reference <- Start(dat = path,
                   var = var,
                   year = paste0(years_hist),
                   month = 'all',
                   lat = values(list(lat_min, lat_max)),
                   lon = values(list(lon_min, lon_max)),
                   synonims = list(lat = c('lat','latitude'), 
                                   lon = c('lon','longitude'),
                                   month = c('month','time')),
                   return_vars = list(lat = NULL, lon = NULL),
                   lat_reorder = Sort(decreasing = F),
                   lon_reorder = CircularSort(0, 360),
                   num_procs = 1, retrieve = FALSE)

all(attributes(hist)$Variables$common$lat == attributes(exp)$Variables$common$lat)
all(attributes(reference)$Variables$common$lat == attributes(hist)$Variables$common$lat)

